from pyfiglet import figlet_format
import textwrap

APP_NAME = "pyQA"
LOGO = figlet_format(APP_NAME, font="slant")

DEFUALT_QUERY = "how to center a div"

EPILOG = textwrap.dedent(
    """\
                                    environment variable examples:
                                    PYQA_COLORIZE=1
                                    PYQA_DISABLE_CACHE=1
                                    PYQA_DISABLE_SSL=1
                                    PYQA_SEARCH_ENGINE=google
                                    PYQA_URL=serverfault.com
                                    """
)
# variables for text formatting, prepend to string to begin text formatting.
BOLD = "\033[1m"
GREEN = "\033[92m"
RED = "\033[91m"
UNDERLINE = "\033[4m"
END_FORMAT = "\033[0m"  # append to string to end text formatting.

NO_RESULTS_MESSAGE = "Sorry, couldn't find any help with that topic"
NO_ANSWER_MSG = "< no answer given >"

# stash options
STASH_SAVE = "save"
STASH_VIEW = "view"
STASH_REMOVE = "remove"
STASH_EMPTY = "empty"

BLOCKED_ENGINES = []

SCHEME = "https://"
VERIFY_SSL_CERTIFICATE = True

SUPPORTED_SEARCH_ENGINES = ("google", "bing", "duckduckgo")

DEFAULT_URL = "stackoverflow.com"

USER_AGENTS = (
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:11.0) Gecko/20100101 Firefox/11.0",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:22.0) Gecko/20100 101 Firefox/22.0",
    "Mozilla/5.0 (Windows NT 6.1; rv:11.0) Gecko/20100101 Firefox/11.0",
    (
        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.5 (KHTML, like Gecko) "
        "Chrome/19.0.1084.46 Safari/536.5"
    ),
    (
        "Mozilla/5.0 (Windows; Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46"
        "Safari/536.5"
    ),
)

SEARCH_URLS = {
    "bing": SCHEME + "www.bing.com/search?q=site:{0}%20{1}&hl=en",
    "google": SCHEME + "www.google.com/search?q=site:{0}%20{1}&hl=en",
    "duckduckgo": SCHEME + "duckduckgo.com/html?q=site:{0}%20{1}&t=hj&ia=web",
}

BLOCK_INDICATORS = (
    'form id="captcha-form"',
    "This page appears when Google automatically detects requests coming from your computer "
    'network which appear to be in violation of the <a href="//www.google.com/policies/terms/">Terms of Service',
)

CACHE_EMPTY_VAL = "NULL"
CACHE_ENTRY_MAX = 128

URL = "https://stackoverflow.com/questions/64007220/how-to-use-django-filters-to-filter-for-many-values-for-the-same-field"



"""Console script for pyqa."""
import sys

import click
from rich import print
from rich.align import Align
from rich.panel import Panel
from rich.text import Text

from pyqa import __version__
from pyqa import utils
from pyqa.constants import LOGO, URL, DEFUALT_QUERY, EPILOG


@click.group()  # invoke_without_command=True
@click.version_option(message="pyQA, version %(version)s")
@click.option("-a", "--all", is_flag=True)
"-f", "--filename", "dest", the name is dest
@click.option(
    "-e",
    "--engine",
    help="search engine for this query (google, bing, duckduckgo)",
    default="bing",
)
@click.pass_context
def main(ctx, **kwargs):
    ctx.ensure_object(dict)
    args = ctx.obj

    query = kwargs.get("query", "yusufadell")
    args["engine"] = kwargs["engine"]
    args["all"] = kwargs["all"]

"-f", "-fb", the name is f

"--f", "--foo-bar", the name is f

@main.command(epilog=EPILOG)
@click.argument("query", nargs=-1)
@click.pass_context
def query(ctx, **kwargs):
    args = ctx.obj
    query = kwargs["query"] or DEFUALT_QUERY
    args["query"] = " ".join(query)

    print(
        Panel(
            Align.center(
                Text.from_ansi(LOGO, no_wrap=True),
                vertical="middle",
            ),
            border_style="green",
            title="pyQA",
            subtitle="Thank you for using pyQA",
)
    )

    url = URL  # TODO: get link from query
    res = utils._get_answer(args, url)
    answer = res
    print(
        Panel(
            Align.center(
                Text.from_ansi(answer, no_wrap=True),
                vertical="middle",
            ),
            border_style="green",
            title="pyQA",
            subtitle="Tank you for using pyQA",
)
    )


@main.command()
@click.pass_context
def run(ctx):
    from .pyqa import pyqa

    pyqa_result = pyqa(ctx)
    # if os.name == "nt":
    #     # Windows
    #     print(pyqa_result)
    # else:
    #     utf8_result = pyqa_result.encode("utf-8", "ignore")
    #     # Write UTF-8 to stdout
    #     click.echo(utf8_result)
    click.echo(pyqa_result)


# If you have a subcommand called run taking an option called reload
# and the prefix is WEB, then the variable is WEB_RUN_RELOAD.
if __name__ == "__main__":
    main(obj={})  # pragma: no cover



import inspect
import logging
import os
from urllib.request import getproxies

import appdirs
import requests

from cachelib import FileSystemCache, NullCache
from pyqa.constants import (APP_NAME, CACHE_ENTRY_MAX, END_FORMAT, GREEN,
                            NO_ANSWER_MSG, RED, SUPPORTED_SEARCH_ENGINES,
                            USER_AGENTS, VERIFY_SSL_CERTIFICATE)
from pyqa.erros import (BingValidationError, DDGValidationError,
                        GoogleValidationError)

# =======================================
#         get page content             ||
# =======================================

pyqa_session = requests.session()


def get_proxies():
    proxies = getproxies()
    filtered_proxies = {}
    for key, value in proxies.items():
        if key.startswith("http"):
            if not value.startswith("http"):
                filtered_proxies[key] = f"http://{value}"
            else:
                filtered_proxies[key] = value
    return filtered_proxies


def _random_int(width):
    bres = os.urandom(width)
    import sys

    if sys.version < "3":
        ires = int(bres.encode("hex"), 16)
    else:
        ires = int.from_bytes(bres, "little")

    return ires


def _random_choice(seq):
    return seq[_random_int(1) % len(seq)]


def _get_result(url):
    try:
        resp = pyqa_session.get(
            url,
            headers={"User-Agent": _random_choice(USER_AGENTS)},
            proxies=get_proxies(),
            verify=VERIFY_SSL_CERTIFICATE,
            cookies={"CONSENT": "YES+US.en+20221119-00-0"},
        )
        resp.raise_for_status()
        return resp.text
    except requests.exceptions.SSLError as error:
        logging.error(
            "%sEncountered an SSL Error. Try using HTTP instead of "
            'HTTPS by setting the environment variable "PYQA_DISABLE_SSL".\n%s',
            RED,
            END_FORMAT,
        )
        raise error


# =======================================
#            extract answer            ||
# =======================================
from pyquery import PyQuery as pq


def _add_links_to_text(element):
    hyperlinks = element.find("a")

    for hyperlink in hyperlinks:
        pquery_object = pq(hyperlink)
        href = hyperlink.attrib["href"]
        copy = pquery_object.text()
        if copy == href:
            replacement = copy
        else:
            replacement = f"[{copy}]({href})"
        pquery_object.replace_with(replacement)


def get_text(element):
    """return inner text in pyquery element"""
    _add_links_to_text(element)
    try:
        return element.text(squash_space=False)
    except TypeError:
        return element.text()


def _get_answer(args, url):  # pylint: disable=too-many-branches
    logging.info("Fetching page: %s", url)
    page = _get_result(url + "?answertab=votes")

    html = pq(page)

    first_answer = html(".answercell").eq(0) or html(".answer").eq(0)

    instructions = first_answer.find("pre") or first_answer.find("code")
    args["tags"] = [t.text for t in html(".post-tag")]
    # make decision on answer body class.
    if first_answer.find(".js-post-body"):
        answer_body_cls = ".js-post-body"
    else:
        # rollback to post-text class
        answer_body_cls = ".post-text"

    if not instructions and not args["all"]:
        logging.info("No code sample found, returning entire answer")
        text = get_text(first_answer.find(answer_body_cls).eq(0))
    elif args["all"]:
        logging.info("Returning entire answer")
        texts = []
        for html_tag in first_answer.items(f"{answer_body_cls} > *"):
            current_text = get_text(html_tag)
            if current_text:
                if html_tag[0].tag in ["pre", "code"]:
                    texts.append(current_text)
                else:
                    texts.append(current_text)
        text = "\n".join(texts)
    else:
        text = get_text(instructions.eq(0))
    if text is None:
        logging.info("%sAnswer was empty%s", RED, END_FORMAT)
        text = NO_ANSWER_MSG
    text = text.strip()
    return text




"""Main module."""
import os


from .constants import DEFAULT_URL

get_env = os.getenv
URL = get_env("PYQA_URL", DEFAULT_URL)


def command_line_runner():
    """Command line runner."""
    print(f"pyQA Runnser")


if __name__ == "__main__":
    command_line_runner()



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages
from pyqa import __version__, __author__, __email__

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = [
    "Click>=7.0",
    "rich>=12.6.0",
    "textual>=0.4.0",
]

test_requirements = [
    "pytest>=3",
]

setup(
    author=__author__,
    author_email=__email__,
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="Python console appliction that searches the internet for asnwers to coding problems",
    entry_points={
        "console_scripts": [
            "pyqa=pyqa.cli:main",
        ],
    },
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="pyqa",
    name="pyqa-y8l",
    packages=find_packages(include=["pyqa", "pyqa.*"]),
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/yusufadell/pyqa",
    version=__version__,
    zip_safe=False,
)
